import numpy as np
import networkx as nx
import random
import copy
import pandas as pd
import scipy.stats as stats

from sabcom.agent import Agent
from sabcom.helpers import what_coordinates, what_informality


class Environment:
    """
    The environment class contains the agents in a network structure
    """

    def __init__(self, seed, parameters, district_data, age_distribution_per_district,
                 household_contact_matrix, other_contact_matrix, household_size_distribution, travel_matrix):
        """
        This method initialises the environment and its properties.

        :param seed: used to initialise the random generators to ensure reproducibility, int
        :param parameters: contains all model parameters, dictionary
        :param district_data: contains empirical data on the districts, list of tuples
        :param age_distribution_per_district: contains the distribution across age categories per district, dictionary
        :param household_contact_matrix: contains number and age groups for household contacts, Pandas DataFrame
        :param other_contact_matrix: contains number and age groups for all other contacts, Pandas DataFrame
        :param household_size_distribution: contains distribution of household size for all districts, Pandas DataFrame
        :param travel_matrix: contains number and age groups for all other contacts, Pandas DataFrame
        """
        np.random.seed(seed)
        random.seed(seed)
        random.seed(seed)

        self.parameters = parameters
        self.other_contact_matrix = other_contact_matrix

        # 1 create modelled districts
        # 1.1 retrieve population data
        nbd_values = [x[1] for x in district_data]
        population_per_neighbourhood = [x['Population'] for x in nbd_values]

        # 1.2 correct the population in districts to be proportional to number of agents
        correction_factor = sum(population_per_neighbourhood) / parameters["number_of_agents"]
        corrected_populations = [int(round(x / correction_factor)) for x in population_per_neighbourhood]

        # 1.3 only count districts that then have an amount of people bigger than 0
        indices_big_neighbourhoods = [i for i, x in enumerate(corrected_populations) if x > 0]
        corrected_populations_final = [x for i, x in enumerate(corrected_populations) if x > 0]

        # 1.4 fill up the districts with agents
        self.districts = [x[0] for x in district_data]
        self.district_agents = {d: [] for d in self.districts}
        agents = []
        city_graph = nx.Graph()
        agent_name = 0
        all_travel_districts = {district_data[idx][0]: [] for idx in indices_big_neighbourhoods}
        for num_agents, idx in zip(corrected_populations_final, indices_big_neighbourhoods):
            # 1.4.1 determine district code, informality, and age categories
            district_list = []
            district_code = district_data[idx][0]
            informality = what_informality(district_code, district_data) * parameters["informality_dummy"]

            age_categories = np.random.choice(age_distribution_per_district[district_code].index,
                                              size=int(num_agents),
                                              replace=True,
                                              p=age_distribution_per_district[district_code].values)

            # 1.4.2 determine district to travel to
            available_districts = list(all_travel_districts.keys())
            probabilities = list(travel_matrix[[str(x) for x in available_districts]].loc[district_code])

            # 1.4.3 add agents to district
            lower, upper = -(parameters['stringency_index'][0] / 100), (1 - (parameters['stringency_index'][0] / 100))
            mu, sigma = 0.0, parameters['private_shock_stdev']
            truncnorm_shock_generator = stats.truncnorm((lower - mu) / sigma, (upper - mu) / sigma, loc=mu, scale=sigma)

            for a in range(num_agents):
                init_private_signal = parameters['stringency_index'][0] / 100 + truncnorm_shock_generator.rvs(1)[0]
                district_to_travel_to = np.random.choice(available_districts, size=1, p=probabilities)[0]
                agent = Agent(agent_name, 's',
                              district_code,
                              age_categories[a],
                              informality,
                              int(round(other_contact_matrix.loc[age_categories[a]].sum())),
                              district_to_travel_to,
                              init_private_signal #TODO debug!
                              )
                self.district_agents[district_code].append(agent)
                district_list.append(agent)
                all_travel_districts[district_to_travel_to].append(agent)
                agent_name += 1

            # 2 Create the household network structure
            # 2.1 get household size list for this Ward and reduce list to max household size = size of ward
            max_district_household = min(len(district_list), len(household_size_distribution.columns) - 1)
            hh_sizes = household_size_distribution.loc[district_code][:max_district_household]
            # 2.2 then calculate probabilities of this being of a certain size
            hh_probability = pd.Series([float(i) / sum(hh_sizes) for i in hh_sizes])
            hh_probability.index = hh_sizes.index
            # 2.3 determine household sizes
            sizes = []
            while sum(sizes) < len(district_list):
                sizes.append(int(np.random.choice(hh_probability.index, size=1, p=hh_probability)[0]))
                hh_probability = hh_probability[:len(district_list) - sum(sizes)]
                # recalculate probabilities
                hh_probability = pd.Series([float(i) / sum(hh_probability) for i in hh_probability])
                hh_probability.index = hh_sizes.index[:len(district_list) - sum(sizes)]

            # 2.4 Distribute agents over households
            # 2.4.1 pick the household heads and let it form connections with other based on probabilities.
            # household heads are chosen at random without replacement
            household_heads = np.random.choice(district_list, size=len(sizes), replace=False)
            not_household_heads = [x for x in district_list if x not in household_heads]
            # 2.4.2 let the household heads pick n other agents that are not household heads themselves
            for i, head in enumerate(household_heads):
                head.household_number = i
                if sizes[i] > 1:
                    # pick n other agents based on probability given their age
                    p = [household_contact_matrix[to.age_group].loc[head.age_group] for to in not_household_heads]
                    # normalize p
                    p = [float(i) / sum(p) for i in p]
                    household_members = list(np.random.choice(not_household_heads, size=sizes[i]-1, replace=False, p=p))

                    # remove household members from not_household_heads
                    for h in household_members:
                        h.household_number = i
                        not_household_heads.remove(h)

                    # add head to household members:
                    household_members.append(head)
                else:
                    household_members = [head]

                # 2.4.3 create graph for household
                household_graph = nx.Graph()
                household_graph.add_nodes_from(range(len(household_members)))

                # create edges between all household members
                edges = nx.complete_graph(len(household_members)).edges()
                household_graph.add_edges_from(edges, label='household')

                # add household members to the agent list
                agents.append(household_members)

                # 2.4.4 add network to city graph
                city_graph = nx.disjoint_union(city_graph, household_graph)

        self.agents = [y for x in agents for y in x]

        # 3 Next, we create the a city wide network structure of recurring contacts based on the travel matrix
        for agent in self.agents:
            agents_to_travel_to = all_travel_districts[agent.district_to_travel_to]
            agents_to_travel_to.remove(agent)  # remove the agent itself

            if agents_to_travel_to:
                # select the agents which it is most likely to have contact with based on the travel matrix
                p = [other_contact_matrix[a.age_group].loc[agent.age_group] for a in agents_to_travel_to]
                # normalize p
                p = [float(i) / sum(p) for i in p]

                location_closest_agents = np.random.choice(agents_to_travel_to,
                                                           size=min(agent.num_contacts, len(agents_to_travel_to)),
                                                           replace=False,
                                                           p=p)

                for ca in location_closest_agents:
                    city_graph.add_edge(agent.name, ca.name, label='other')

        self.network = city_graph

        # rename agents to reflect their new position
        for idx, agent in enumerate(self.agents):
            agent.name = idx

        # add agent to the network structure
        for idx, agent in enumerate(self.agents):
            self.network.nodes[idx]['agent'] = agent

        self.infection_states = []
        self.infection_quantities = {key: [] for key in ['e', 's', 'i1', 'i2', 'c', 'r', 'd', 'compliance']}

        # NEW feature added stringency index
        self.stringency_index = parameters['stringency_index']
        if len(parameters['stringency_index']) < parameters['time']:
            self.stringency_index += [parameters['stringency_index'][-1] for x in range(len(
                parameters['stringency_index']), parameters['time'])]

    def store_network(self):
        """Returns a deep copy of the current network"""
        current_network = copy.deepcopy(self.network)
        return current_network

    def write_status_location(self, period, seed, base_folder='measurement/'):
        """
        Writes information about the agents and their status in the current period to a csv file

        :param period: the current time period, int
        :param seed: used to initialise the random generators to ensure reproducibility, int
        :param base_folder: the location of the folder to write the csv to, string
        :return: None
        """
        location_status_data = {'agent': [], 'status': [], 'WardID': [], 'age_group': [],
                                'others_infected': [], 'compliance': []}
        for agent in self.agents:
            location_status_data['agent'].append(agent.name)
            location_status_data['status'].append(agent.status)
            location_status_data['WardID'].append(agent.district)
            location_status_data['age_group'].append(agent.age_group)
            location_status_data['others_infected'].append(agent.others_infected)
            location_status_data['compliance'].append(agent.compliance)

        pd.DataFrame(location_status_data).to_csv(base_folder + "seed" + str(seed) + "/agent_data{0:04}.csv".format(
            period))

        # output links
        if period == 0:
            pd.DataFrame(self.network.edges()).to_csv(base_folder + "seed" + str(seed) + "/edge_list{0:04}.csv".format(
                period))



import random
import numpy as np
import pandas as pd
import math
from sklearn import preprocessing
import scipy.stats as stats


def edge_in_cliq(edge, nodes_in_cliq):
    if edge[0] in nodes_in_cliq:
        return True
    else:
        return False


def edges_to_remove_neighbourhood(all_edges, neighbourhood_density, nbh_nodes):
    neighbourhood_edges = [e for e in all_edges if edge_in_cliq(e, nbh_nodes)]
    sample_size = int(len(neighbourhood_edges) * (1-neighbourhood_density))
    # sample random edges
    chosen_edges = random.sample(neighbourhood_edges, sample_size)
    return chosen_edges


def what_neighbourhood(index, neighbourhood_nodes):
    for n in neighbourhood_nodes:
        if index in neighbourhood_nodes[n]:
            return n

    raise ValueError('Neighbourhood not found.')


def what_coordinates(neighbourhood_name, dataset):
    for x in range(len(dataset)):
        if neighbourhood_name in dataset[x]:
            return dataset[x][1]['lon'], dataset[x][1]['lat'],

    raise ValueError("Corresponding coordinates not found")


def what_informality(neighbourhood_name, dataset):
    for x in range(len(dataset)):
        if neighbourhood_name in dataset[x]:
            try:
                return dataset[x][1]['Informal_residential']
            except:
                return None

    raise ValueError("Corresponding informality not found")


def confidence_interval(data, av):
    sample_stdev = np.std(data)
    sigma = sample_stdev/math.sqrt(len(data))
    return stats.t.interval(alpha=0.95, df=24, loc=av, scale=sigma)


def generate_district_data(number_of_agents, path, max_districts=None):
    """
    Transforms input data on informal residential, initial infections, and population and transforms it to
    a list of organised data for the simulation.

    :param number_of_agents: number of agents in the simulation, integer
    :param max_districts: (optional) maximum amount of districts simulated, integer
    :return: data set containing district data for simulation, list
    """
    informal_residential = pd.read_csv('{}/f_informality.csv'.format(path))#.iloc[:-1]
    inital_infections = pd.read_csv('{}/f_initial_cases.csv'.format(path), index_col=1)
    inital_infections = inital_infections.sort_index()
    population = pd.read_csv('{}/f_population.csv'.format(path))

    # normalise district informality
    x = informal_residential[['Informal_residential']].values.astype(float)
    min_max_scaler = preprocessing.MinMaxScaler()
    x_scaled = min_max_scaler.fit_transform(x)
    informal_residential['Informal_residential'] = pd.DataFrame(x_scaled)
    population['Informal_residential'] = informal_residential['Informal_residential']

    # determine smallest district based on number of agents
    smallest_size = population['Population'].sum() / number_of_agents

    # generate data set for model input
    districts_data = []
    for i in range(len(population)):
        if population['Population'].iloc[i] > smallest_size:
            districts_data.append(
                [int(population['WardID'].iloc[i]), {'Population': population['Population'].iloc[i],
                                                     #'lon': population['lon'].iloc[i],
                                                     #'lat': population['lat'].iloc[i],
                                                     'Informal_residential': population['Informal_residential'].iloc[i],
                                                     'Cases_With_Subdistricts':
                                                         inital_infections.loc[population['WardID'].iloc[i]][
                                                             'Cases'],
                                                     },
                 ])

    if max_districts is None:
        max_districts = len(districts_data)  # this can be manually shortened to study dynamics in some districts

    return districts_data[:max_districts]



import numpy as np


def differential_equations_model(compartments, t, infection_rate, contact_probability_matrix,
                                 exit_rate_exposed, exit_rate_asymptomatic,
                                 exit_rate_symptomatic, exit_rate_critical,
                                 probability_symptomatic, probability_critical, probability_to_die, hospital_capacity):
    # reshape 63 element vector Z into [7 x 9] matrix
    compartments = compartments.reshape(7, -1)

    # assign rows to disease compartments
    susceptible, exposed, asymptomatic, symptomatic, critical, recovered, dead = compartments

    health_overburdened_multiplier = 1

    # health system can be overburdened which will increase the probability of death
    if critical.sum() > hospital_capacity:
        health_overburdened_multiplier = 1.79 #TODO add this as a parameter
        probability_to_die = np.minimum(health_overburdened_multiplier * probability_to_die, np.ones(9))
        # print(t)

    # construct differential equation evolution equations
    delta_susceptible = -infection_rate * susceptible * contact_probability_matrix.dot((asymptomatic + symptomatic))
    delta_exposed = infection_rate * susceptible * contact_probability_matrix.dot((
            asymptomatic + symptomatic)) - exit_rate_exposed * exposed
    delta_asymptomatic = (1 - probability_symptomatic
                          ) * exit_rate_exposed * exposed - exit_rate_asymptomatic * asymptomatic
    delta_symptomatic = probability_symptomatic * exit_rate_exposed * exposed - exit_rate_symptomatic * symptomatic
    delta_critical = probability_critical * exit_rate_symptomatic * symptomatic - exit_rate_critical * critical
    delta_recovered = exit_rate_asymptomatic * asymptomatic + (
            1 - probability_critical) * exit_rate_symptomatic * symptomatic + (1 - probability_to_die
                                                                               ) * exit_rate_critical * critical
    delta_dead = probability_to_die * exit_rate_critical * critical

    # store differentials as 63 element vector
    delta_compartments = np.concatenate((delta_susceptible, delta_exposed, delta_asymptomatic,
                                         delta_symptomatic, delta_critical, delta_recovered, delta_dead), axis=0)

    return delta_compartments



import sys
import click
import pickle
import os
import time
import logging
import json
import pandas as pd
import networkx as nx
import numpy as np
from scipy.integrate import odeint

from sabcom.runner import runner
from sabcom.differential_equation_model import differential_equations_model
from sabcom.environment import Environment
from sabcom.helpers import generate_district_data, what_informality


@click.group()
@click.version_option("1.0.0")
def main():
    """
    An open source, easy-to-use-and-adapt, spatial network, multi-agent, model
    that can be used to simulate the effects of different lockdown policy measures
    on the spread of the Covid-19 virus in several (South African) cities.
    """
    pass


@main.command()
@click.option('--input_folder_path', '-i', type=click.Path(exists=True), required=True,
              help="This should contain all necessary input files, specifically an initialisation folder")
@click.option('--output_folder_path', '-o', type=click.Path(exists=True), required=True,
              help="All simulation output will be deposited here")
@click.option('--seed', '-s', type=int, required=True,
              help="Integer seed number that is used for Monte Carlo simulations")
@click.option('--data_output_mode', '-d', default='csv-light', show_default=True,
              type=click.Choice(['csv-light', 'csv', 'network'],  case_sensitive=False,))
@click.option('--scenario', '-sc', default='no-intervention', show_default=True,
              type=click.Choice(['no-intervention', 'lockdown', 'ineffective-lockdown'],  case_sensitive=False,))
@click.option('--days', '-day', default=None, type=int, required=False,
              help="change the number of simulation days here with the caveat that simulation time can only be shortened compared to what was initialised.")
@click.option('--probability_transmission', '-pt', default=None, type=float, required=False,
              help="change the probability of transmission between two agents.")
@click.option('--visiting_recurring_contacts_multiplier', '-cont', default=None, type=float, required=False,
              help="change the percentage of contacts agent may have.")
@click.option('--likelihood_awareness', '-la', default=None, type=float, required=False,
              help="change the likelihood that an agent is aware it is infected.")
@click.option('--gathering_max_contacts', '-maxc', default=None, type=int, required=False,
              help="change maximum number of contacts and agent is allowed to have.")
@click.option('--initial_infections', '-ini', default=None, type=int, required=False,
              help="number of initial infections")
@click.option('--sensitivity_config_file_path', '-scf', type=click.Path(exists=True), required=False,
              help="Config file that contains parameter combinations for sensitivity analysis on HPC")
def simulate(**kwargs):
    """Simulate the model"""
    start = time.time()

    # format arguments
    seed = kwargs.get('seed')
    output_folder_path = kwargs.get('output_folder_path')

    # logging initialisation
    logging.basicConfig(filename=os.path.join(output_folder_path,
                                              'simulation_seed{}.log'.format(seed)), filemode='w', level=logging.DEBUG)

    input_folder_path = kwargs.get('input_folder_path')
    inititialisation_path = os.path.join(input_folder_path, 'initialisations')
    seed_path = os.path.join(inititialisation_path, 'seed_{}.pkl'.format(seed))
    logging.info('Start of simulation seed{} with arguments -i ={}, -o={}'.format(seed,
                                                                                  input_folder_path,
                                                                                  output_folder_path))

    if not os.path.exists(seed_path):
        click.echo(seed_path + ' not found', err=True)
        click.echo('Error: specify a valid seed')
        return

    data = open(seed_path, "rb")
    list_of_objects = pickle.load(data)
    environment = list_of_objects[0]

    # update optional parameters
    if kwargs.get('days'):
        environment.parameters['time'] = kwargs.get('days')
        # add line to expand stringency index
        click.echo('Time has been set to {}'.format(environment.parameters['time']))
        logging.debug('Time has been set to {}'.format(environment.parameters['time']))
        # ensure that stringency is never shorter than time if time length is increased
        if len(environment.stringency_index) < environment.parameters['time']:
            environment.stringency_index += [environment.stringency_index[-1] for x in range(
                len(environment.stringency_index), environment.parameters['time'])]
        logging.debug('The stringency index has been lenghtened by {}'.format(
            environment.parameters['time'] - len(environment.stringency_index)))

    if kwargs.get('probability_transmission'):
        environment.parameters['probability_transmission'] = kwargs.get('probability_transmission')
        click.echo('Transmission probability has been set to {}'.format(environment.parameters['probability_transmission']))
        logging.debug('Transmission probability has been set to {}'.format(environment.parameters['probability_transmission']))

    if kwargs.get('visiting_recurring_contacts_multiplier'):
        environment.parameters['visiting_recurring_contacts_multiplier'] = [kwargs.get('visiting_recurring_contacts_multiplier') for x in range(environment.parameters['time'])]
        click.echo('Recurring contacts has been set to {}'.format(environment.parameters['visiting_recurring_contacts_multiplier'][0]))
        logging.debug(
            'Recurring contacts has been set to {}'.format(environment.parameters['visiting_recurring_contacts_multiplier'][0]))

    if kwargs.get('likelihood_awareness'):
        environment.parameters['likelihood_awareness'] = kwargs.get('likelihood_awareness')
        click.echo('Likelihood awareness has been set to {}'.format(environment.parameters['likelihood_awareness']))
        logging.debug(
            'Likelihood awareness has been set to {}'.format(environment.parameters['likelihood_awareness']))

    if kwargs.get('gathering_max_contacts'):
        environment.parameters['gathering_max_contacts'] = kwargs.get('gathering_max_contacts')
        click.echo('Max contacts has been set to {}'.format(environment.parameters['gathering_max_contacts']))
        logging.debug(
            'Max contacts has been set to {}'.format(environment.parameters['gathering_max_contacts']))

    if kwargs.get('initial_infections'):
        environment.parameters['initial_infections'] = [x for x in range(round(int(kwargs.get('initial_infections'))))]
        click.echo('Initial infections have been set to {}'.format(len(environment.parameters['initial_infections'])))
        logging.debug('Initial infections have been set to {}'.format(len(environment.parameters['initial_infections'])))

    if kwargs.get('sensitivity_config_file_path'):
        # open file
        config_path = kwargs.get('sensitivity_config_file_path')
        if not os.path.exists(config_path):
            click.echo(config_path + ' not found', err=True)
            click.echo('Error: specify a valid path to the sensitivity config file')
            return
        else:
            with open(config_path) as json_file:
                config_file = json.load(json_file)

                for param in config_file:
                    environment.parameters[param] = config_file[param]

    # transform input data to general district data for simulations
    district_data = generate_district_data(environment.parameters['number_of_agents'], path=input_folder_path)

    # set scenario specific parameters
    scenario = kwargs.get('scenario', 'no-intervention')
    print('scenario is ', scenario)
    if scenario == 'no-intervention':
        environment.parameters['likelihood_awareness'] = 0.0
        environment.parameters['visiting_recurring_contacts_multiplier'] = [
            1.0 for x in environment.parameters['visiting_recurring_contacts_multiplier']]
        environment.parameters['gathering_max_contacts'] = float('inf')
        environment.parameters['physical_distancing_multiplier'] = 1.0
        environment.parameters['informality_dummy'] = 0.0
    elif scenario == 'lockdown':
        environment.parameters['informality_dummy'] = 0.0
    elif scenario == 'ineffective-lockdown':
        environment.parameters['informality_dummy'] = 1.0

    # log parameters used after scenario called
    for param in environment.parameters:
        logging.debug('Parameter {} has the value {}'.format(param, environment.parameters[param]))

    for agent in environment.agents:
        agent.informality = what_informality(agent.district, district_data
                                             ) * environment.parameters["informality_dummy"]

    initial_infections = pd.read_csv(os.path.join(input_folder_path, 'f_initial_cases.csv'), index_col=0)

    # save csv light or network data
    data_output_mode = kwargs.get('data_output_mode', 'csv-light')  # default output mode is csv_light
    environment.parameters["data_output"] = data_output_mode

    environment = runner(environment=environment, initial_infections=initial_infections, seed=int(seed),
                         data_folder=output_folder_path,
                         data_output=data_output_mode)

    if data_output_mode == 'network':
        for idx, network in enumerate(environment.infection_states):
            for i, node in enumerate(network.nodes):
                network.nodes[i]['agent'] = network.nodes[i]['agent'].status
            idx_string = '{0:04}'.format(idx)
            nx.write_graphml(network, os.path.join(output_folder_path, "seed{}network_time{}.graphml".format(seed,
                                                                                                              idx_string)))
    elif data_output_mode == 'csv-light':
        pd.DataFrame(environment.infection_quantities).to_csv(os.path.join(output_folder_path,
                                                                           'seed{}quantities_state_time.csv'.format(seed)))

    end = time.time()
    hours_total, rem_total = divmod(end - start, 3600)
    minutes_total, seconds_total = divmod(rem_total, 60)
    click.echo("TOTAL RUNTIME {:0>2}:{:0>2}:{:05.2f}".format(int(hours_total), int(minutes_total), seconds_total))
    click.echo('Simulation done, check out the output data here: {}'.format(output_folder_path))


@main.command()
@click.option('--input_folder_path', '-i', type=click.Path(exists=True), required=True,
              help="Folder containing parameters file, input data and an empty initialisations folder")
@click.option('--seed', '-s', type=int, required=True,
              help="Integer seed number that is used for Monte Carlo simulations")
@click.option('--output_folder_path', '-o', type=click.Path(exists=True), required=True,
              help="All simulation output will be deposited here")
@click.option('--output_folder_path', '-o', type=click.Path(exists=True), required=True)
def initialise(**kwargs):  # input output seed
    """Initialise the model in specified directory"""
    start = time.time()
    seed = kwargs.get('seed')

    input_folder_path = kwargs.get('input_folder_path')

    # format optional arguments
    parameters_path = os.path.join(input_folder_path, 'parameters.json')
    initialisations_folder_path = os.path.join(input_folder_path, 'initialisations')

    # logging initialisation
    logging.basicConfig(filename=os.path.join(initialisations_folder_path,
                                              'initialise_seed{}.log'.format(seed)), filemode='w', level=logging.DEBUG)
    logging.info('Start of initialisation seed{} with arguments -i ={}'.format(seed, input_folder_path))

    if not os.path.exists(initialisations_folder_path):
        click.echo(initialisations_folder_path + ' not found', err=True)
        click.echo('No initialisation folder to place initialisation pickle')
        return

    if not os.path.exists(parameters_path):
        click.echo(parameters_path + ' not found', err=True)
        click.echo('No parameter file found')
        return

    with open(parameters_path) as json_file:
        parameters = json.load(json_file)
        for param in parameters:
            logging.debug('Parameter {} is {}'.format(param, parameters[param]))

    # Change parameters depending on experiment
    #data_output_mode = kwargs.get('data_output_mode', 'csv-light')  # TODO is this still needed?

    #parameters['data_output'] = data_output_mode

    age_groups = ['age_0_10', 'age_10_20', 'age_20_30', 'age_30_40', 'age_40_50',
                  'age_50_60', 'age_60_70', 'age_70_80', 'age_80_plus']

    # 2 load district data
    # transform input data to general district data for simulations
    district_data = generate_district_data(int(parameters['number_of_agents']), path=input_folder_path)

    # 2.2 age data
    age_distribution = pd.read_csv(os.path.join(input_folder_path, 'f_age_distribution.csv'), sep=',', index_col=0)
    age_distribution_per_ward = dict(age_distribution.transpose())

    # 2.3 household size distribution
    HH_size_distribution = pd.read_csv(os.path.join(input_folder_path, 'f_household_size_distribution.csv'), index_col=0)

    # 3 load travel matrix
    travel_matrix = pd.read_csv(os.path.join(input_folder_path, 'f_travel.csv'), index_col=0)

    # 4 load contact matrices
    # 4.1 load household contact matrix
    hh_contact_matrix = pd.read_csv(os.path.join(input_folder_path, 'f_household_contacts.csv'), index_col=0)
    # add a column & row for 80 plus. Rename columns to match our age categories
    hh_contact_matrix['80plus'] = hh_contact_matrix['70_80']
    row = hh_contact_matrix.xs('70_80')
    row.name = '80plus'
    hh_contact_matrix = hh_contact_matrix.append(row)
    hh_contact_matrix.columns = age_groups
    hh_contact_matrix.index = age_groups

    # 4.2 load other contact matrix
    other_contact_matrix = pd.read_csv(os.path.join(input_folder_path, 'f_nonhousehold_contacts.csv'), index_col=0)
    other_contact_matrix['80plus'] = other_contact_matrix['70_80']
    row = other_contact_matrix.xs('70_80')
    row.name = '80plus'
    other_contact_matrix = other_contact_matrix.append(row)
    other_contact_matrix.columns = age_groups
    other_contact_matrix.index = age_groups

    # make new folder if it does not exist
    if not os.path.exists('{}'.format(initialisations_folder_path)):
        os.makedirs('{}'.format(initialisations_folder_path))

    # initialisation
    environment = Environment(int(seed), parameters, district_data, age_distribution_per_ward,
                              hh_contact_matrix, other_contact_matrix, HH_size_distribution, travel_matrix)

    # save environment objects as pickls
    file_name = os.path.join(initialisations_folder_path, "seed_{}.pkl".format(str(seed)))
    save_objects = open(file_name, 'wb')
    pickle.dump([environment, seed], save_objects)
    save_objects.close()

    end = time.time()
    hours_total, rem_total = divmod(end - start, 3600)
    minutes_total, seconds_total = divmod(rem_total, 60)
    click.echo("TOTAL RUNTIME {:0>2}:{:0>2}:{:05.2f}".format(int(hours_total), int(minutes_total), seconds_total))
    click.echo('Initialisation done, check out the output data here: {}'.format(initialisations_folder_path))


@main.command()
@click.option('--input_folder_path', '-i', type=click.Path(exists=True), required=True,
              help="This should contain all necessary input files, specifically a parameter file")
@click.option('--output_folder_path', '-o', type=click.Path(exists=True), required=True,
              help="All simulation output will be deposited here")
@click.option('--r_zero', '-rz', type=float, required=True,
              help="The reproductive rate of the virus in a fully susceptible population")
def demodel(**kwargs):
    input_folder_path = kwargs.get('input_folder_path')
    output_folder_path = kwargs.get('output_folder_path')

    # logging initialisation
    logging.basicConfig(filename=os.path.join(output_folder_path,
                                              'de_model.log'), filemode='w', level=logging.DEBUG)
    logging.info('Start of DE simulation')

    parameters_path = os.path.join(input_folder_path, 'parameters.json')

    if not os.path.exists(parameters_path):
        click.echo(parameters_path + ' not found', err=True)
        click.echo('No parameter file found')
        return

    with open(parameters_path) as json_file:
        parameters = json.load(json_file)
        for param in parameters:
            logging.debug('Parameter {} is {}'.format(param, parameters[param]))

    # arguments = city
    initial_infected = len(parameters['total_initial_infections'])
    T = parameters['time']  # total number of period simulated:

    # Set Covid-19 Parameters:
    # basic reproduction number
    r_zero = kwargs.get('r_zero') #initial_recovered
    exposed_days = float(parameters["exposed_days"])
    asymptomatic_days = float(parameters["asymptom_days"])
    symptomatic_days = float(parameters["symptom_days"])
    critical_days = float(parameters["critical_days"])

    # compartment exit rates
    exit_rate_exposed = 1.0 / exposed_days
    exit_rate_asymptomatic = 1.0 / asymptomatic_days
    exit_rate_symptomatic = 1.0 / symptomatic_days
    exit_rate_critical = 1.0 / critical_days

    probability_symptomatic = parameters["probability_symptomatic"]
    # Probability to become critically ill if symptomatic (source: Verity et al.2020)
    probability_critical = np.array([x for x in parameters["probability_critical"].values()])
    # Probability to die if critically ill (source: Silal et al.2020)
    probability_to_die = np.array([x for x in parameters["probability_critical"].values()])

    # Total population:
    district_population = pd.read_csv(os.path.join(input_folder_path, 'f_population.csv'), index_col=0)
    district_population = district_population.values
    population = district_population.sum()  # sum over wards to obtain city population

    # Set city specific parameters
    hospital_capacity = int(round(parameters["health_system_capacity"] * population))

    # Population by age group (N_age(i) is the population of age group i)
    ward_age_distribution = pd.read_csv(os.path.join(input_folder_path, 'f_age_distribution.csv'),
                                        index_col=0)  # the datafile contains ward level fractions in each age group
    N_age = ward_age_distribution * district_population  # convert to number of people in age group per ward
    N_age = N_age.sum()  # sum over wards
    N_age = N_age.values  # store city level population sizes of each age group

    # Load raw contact matrices
    household_contacts = pd.read_csv(os.path.join(input_folder_path, 'f_household_contacts.csv'), index_col=0)
    other_contacts = pd.read_csv(os.path.join(input_folder_path, 'f_nonhousehold_contacts.csv'), index_col=0)
    contact_matrix = household_contacts + other_contacts
    contact_matrix = contact_matrix.values

    # Replicate last row and column to change the 8 category contact matrix to a 9 category matrix
    contact_matrix = np.vstack((contact_matrix, contact_matrix[7, 0:8]))
    C_last_column = contact_matrix[0:9, 7]
    C_last_column.shape = (9, 1)
    contact_matrix = np.hstack((contact_matrix, C_last_column))

    # Apply reciprocity correction (see Towers and Feng (2012))
    # C_corrected(j,k) = (C(j,k)*N(j) + C(k,j)*N(k))/(2*N(j))
    for j in range(contact_matrix.shape[0]):
        for k in range(contact_matrix.shape[0]):
            contact_matrix[j, k] = (contact_matrix[j, k] * N_age[j] + contact_matrix[k, j] * N_age[k]) / (2 * N_age[j])

    # Scale contact matrix by population size
    # - each column is normalized by the population of that age group: X(i,j)=C(i,j)/N_age(j)
    N_age_row_vector = np.array(N_age)
    N_age_row_vector.shape = (1, 9)
    contact_probability_matrix = np.divide(contact_matrix,
                                           N_age_row_vector)  # X(i,j)=C(i,j)/N_age(j) - entries now measure fraction of each age group contacted on average per day

    # Compute infection_rate from R0, exit_rate_asymptomatic, e_S and dominant eigenvalue of matrix X(i,j)*N_age(i)
    N_age_column_vector = np.array(N_age)
    N_age_column_vector.shape = (9, 1)
    eigen_values, eigen_vectors = np.linalg.eig(np.multiply(contact_probability_matrix, N_age_column_vector))
    dom_eig_val = max(eigen_values)

    infection_rate = (((1 - probability_symptomatic) * exit_rate_asymptomatic + probability_symptomatic * exit_rate_symptomatic) * r_zero) / dom_eig_val
    click.echo('infection rate (beta) is {}'.format(round(infection_rate, 4)))

    # Set initial conditions
    # spread initial infections (exposed individuals) across age groups equally
    initial_exposed = (initial_infected / 9) * np.ones(9)
    # compute remaining initial populations in susceptible compartments
    initial_susceptible = N_age - initial_exposed
    # initiallise other compartments at zero
    initial_asymptomatic = np.zeros(9)
    initial_symptomatic = np.zeros(9)
    initial_critical = np.zeros(9)
    initial_recovered = np.zeros(9)
    initial_dead = np.zeros(9)

    # Solve model over time from initial conditions, using ODE solver from scipy:
    time_points = np.linspace(1, T, T)  # Grid of time points (in days)
    initial_compartments = np.concatenate((initial_susceptible, initial_exposed, initial_asymptomatic,
                                           initial_symptomatic, initial_critical, initial_recovered, initial_dead),
                                          axis=0)

    # Integrate the differential equations over the time grid, t.
    integrals = odeint(differential_equations_model, initial_compartments, time_points, args=(
        infection_rate, contact_probability_matrix, exit_rate_exposed, exit_rate_asymptomatic, exit_rate_symptomatic,
        exit_rate_critical, probability_symptomatic, probability_critical, probability_to_die, hospital_capacity))

    # integrals is T by 63, needs to be split in compartments, each disease compartments has 9 age groups
    susceptible = integrals[:, 0:9].sum(axis=1)
    exposed = integrals[:, 9:18].sum(axis=1)
    asymptomatic = integrals[:, 18:27].sum(axis=1)
    symptomatic = integrals[:, 27:36].sum(axis=1)
    critical = integrals[:, 36:45].sum(axis=1)
    recovered = integrals[:, 45:54].sum(axis=1)
    dead = integrals[:, 54:63].sum(axis=1)

    infected = exposed + asymptomatic + symptomatic + critical + dead + recovered
    active_infections = exposed + asymptomatic + symptomatic + critical
    click.echo('Peak of disease:')
    click.echo('peak critical = {}'.format(round(max(critical))))
    click.echo('peak infected = {}'.format(round(max(active_infections))))
    click.echo('time-period at peak = day {}'.format(np.argmax(active_infections)))
    click.echo('At end of simulation:')
    click.echo('total infected = {} ({} percent of population)'.format(round(infected[T - 1]),
                                                                       round(infected[T - 1] * 100 / population, 2)))
    click.echo('total deceased = {}, ({} percent of infected)'.format(round(dead[T - 1]),
                                                                      round(dead[T - 1] * 100 / infected[T - 1], 2)))

    # export data
    pd.DataFrame({'s': susceptible, 'e': exposed, 'i1': asymptomatic,
                  'i2': symptomatic, 'c': critical, 'r': recovered, 'd': dead}).to_csv(
        os.path.join(output_folder_path, 'DE_quantities_state_time.csv'))

    click.echo('DE model simulation done, check out the output data here: {}'.format(output_folder_path))


if __name__ == '__main__':
    args = sys.argv
    if "--help" in args or len(args) == 1:
        print("SABCoM")
    main()



import numpy as np
import scipy.stats as stats
import random


def runner(environment, initial_infections, seed, data_folder='output_data/',
           data_output=False, calculate_r_naught=False):
    """
    This function is used to run / simulate the model.

    :param environment: contains the parameters and agents, Environment object
    :param initial_infections: contains the Wards and corresponding initial infections, Pandas DataFrame
    :param seed: used to initialise the random generators to ensure reproducibility, int
    :param data_folder:  string of the folder where data output files should be created
    :param data_output:  can be 'csv', 'network', or False (for no output)
    :param calculate_r_naught: set to True to calculate the R0 that the model produces given a single infected agent
    :return: environment object containing the updated agents, Environment object
    """
    # set monte carlo seed
    np.random.seed(seed)
    random.seed(seed)

    # create sets for all agent types
    dead = []
    recovered = []
    critical = []
    sick_with_symptoms = []
    sick_without_symptoms = []
    exposed = []
    susceptible = [agent for agent in environment.agents]
    compliance = []

    # 4 Initialisation of infections
    # here either a fixed initial agent can be infected once to calculate R0
    if calculate_r_naught:
        initial_infected = []
        chosen_agent = environment.agents[environment.parameters['init_infected_agent']]
        chosen_agent.status = 'e'
        initial_infected.append(chosen_agent)
        exposed.append(chosen_agent)
        susceptible.remove(chosen_agent)
    # otherwise infect a set of agents based on the locations of observed infections
    else:
        initial_infections = initial_infections.sort_index()
        cases = [x for x in initial_infections['Cases']]
        probabilities_new_infection_district = [float(i) / sum(cases) for i in cases]

        initial_infected = []
        # select districts with probability
        chosen_districts = list(np.random.choice(environment.districts,
                                                 environment.parameters['total_initial_infections'],
                                                 p=probabilities_new_infection_district))
        # count how often a district is in that list
        chosen_districts = {distr: min(len(environment.district_agents[distr]),
                                       chosen_districts.count(distr)) for distr in chosen_districts}

        for district in chosen_districts:
            # infect appropriate number of random agents
            chosen_agents = np.random.choice(environment.district_agents[district], chosen_districts[district],
                                             replace=False)
            for chosen_agent in chosen_agents:
                chosen_agent.status = 'i2'
                # give i2 days a random value to avoid an unrealistic wave of initial critical cases and deaths
                chosen_agent.sick_days = np.random.randint(0, environment.parameters['symptom_days'])
                sick_with_symptoms.append(chosen_agent)
                susceptible.remove(chosen_agent)

    for t in range(environment.parameters["time"]):
        # Check if the health system is not overburdened
        if len(critical) / len(environment.agents) > environment.parameters["health_system_capacity"]:
            health_overburdened_multiplier = environment.parameters["no_hospital_multiplier"]
        else:
            health_overburdened_multiplier = 1.0

        # create truncnorm generator to generate shocks for this period based on current stringency index
        lower, upper = -(environment.stringency_index[t] / 100), (1 - (environment.stringency_index[t] / 100))
        mu, sigma = 0.0, environment.parameters['private_shock_stdev']
        truncnorm_shock_generator = stats.truncnorm((lower - mu) / sigma, (upper - mu) / sigma, loc=mu, scale=sigma)

        # 5 update status loop
        for agent in susceptible + exposed + sick_without_symptoms + sick_with_symptoms + critical + recovered + dead:
            # construct deGroot signals
            # find indices from neighbour agents
            neighbours_to_learn_from = [environment.agents[x] for x in environment.network.neighbors(agent.name)] # TODO debug!

            private_signal = environment.stringency_index[t] / 100 + truncnorm_shock_generator.rvs(1)[0]
            if neighbours_to_learn_from: # sometimes an agent has no neighbours
                neighbour_signal = np.mean([x.previous_compliance for x in neighbours_to_learn_from])
            else:
                neighbour_signal = private_signal

            agent.compliance = (1 - agent.informality) * \
                               (environment.parameters['weight_private_signal'] * private_signal +
                                (1 - environment.parameters['weight_private_signal']) * neighbour_signal)

            if agent.status == 'e':
                agent.exposed_days += 1
                # some agents will become infectious but do not show agents while others will show symptoms
                if agent.exposed_days > environment.parameters["exposed_days"]:
                    if np.random.random() < environment.parameters["probability_symptomatic"]:
                        agent.status = 'i2'
                        exposed.remove(agent)
                        sick_with_symptoms.append(agent)
                    else:
                        agent.status = 'i1'
                        exposed.remove(agent)
                        sick_without_symptoms.append(agent)

            if agent.status == 'i1':
                agent.asymptomatic_days += 1
                # asymptomatic agents all recover after some time
                if agent.asymptomatic_days > environment.parameters["asymptom_days"]:
                    # calculate R0 here if the first agent recovers
                    if calculate_r_naught and agent in initial_infected:
                        print(t, ' patient zero recovered or dead with R0 = ', agent.others_infects_total)
                        return agent.others_infects_total

                    agent.status = 'r'
                    sick_without_symptoms.remove(agent)
                    recovered.append(agent)

            elif agent.status == 'i2':
                # check if the agent is aware that is is infected here and set compliance to 1.0 if so
                likelihood_awareness = environment.parameters['likelihood_awareness'] * (
                            environment.stringency_index[t] / 100) * (
                                                   1 - agent.informality)  # TODO DEBUG!
                if np.random.random() < likelihood_awareness:
                    agent.compliance = 1.0

                agent.sick_days += 1
                # some symptomatic agents recover
                if agent.sick_days > environment.parameters["symptom_days"]:
                    if np.random.random() < environment.parameters["probability_critical"][agent.age_group]:
                        agent.status = 'c'
                        sick_with_symptoms.remove(agent)
                        critical.append(agent)
                    else:
                        # calculate R0 here if the first agent recovers
                        if calculate_r_naught and agent in initial_infected:
                            print(t, ' patient zero recovered or dead with R0 = ', agent.others_infects_total)
                            return agent.others_infects_total
                        agent.status = 'r'
                        sick_with_symptoms.remove(agent)
                        recovered.append(agent)

            elif agent.status == 'c':
                agent.compliance = 1.0  # TODO debug! make sure that this remains so when dead
                agent.critical_days += 1
                # some agents in critical status will die, the rest will recover
                if agent.critical_days > environment.parameters["critical_days"]:
                    # calculate R0 here if the first agent recovers or dies
                    if calculate_r_naught and agent in initial_infected:
                        print(t, ' patient zero recovered or dead with R0 = ', agent.others_infects_total)
                        return agent.others_infects_total

                    if np.random.random() < (environment.parameters["probability_to_die"][
                                agent.age_group] * health_overburdened_multiplier):
                        agent.status = 'd'
                        critical.remove(agent)
                        dead.append(agent)
                    else:
                        agent.status = 'r'
                        critical.remove(agent)
                        recovered.append(agent)

            elif agent.status == 'r':
                agent.days_recovered += 1
                if np.random.random() < (environment.parameters["probability_susceptible"] * agent.days_recovered):
                    recovered.remove(agent)
                    agent.status = 's'
                    susceptible.append(agent)

        # 6 New infections loop
        for agent in susceptible + exposed + sick_without_symptoms + sick_with_symptoms + critical + recovered + dead:
            agent.others_infected = 0

            # find indices from neighbour agents
            household_neighbours = [x for x in environment.network.neighbors(agent.name) if
                                    environment.agents[x].household_number == agent.household_number and
                                    environment.agents[x].district == agent.district]
            other_neighbours = [x for x in environment.network.neighbors(agent.name) if
                                environment.agents[x].household_number != agent.household_number or
                                environment.agents[x].district != agent.district]

            # depending on compliance, the amount of non-household contacts an agent can visit is reduced
            visiting_r_contacts_multiplier = environment.parameters["visiting_recurring_contacts_multiplier"][t] # TODO debug!
            compliance_term_contacts = (1 - visiting_r_contacts_multiplier) * (1 - agent.compliance)

            # step 1 planned contacts is shaped by
            if other_neighbours:
                planned_contacts = int(round(len(other_neighbours
                                                 ) * (visiting_r_contacts_multiplier + compliance_term_contacts))) # * (visiting_r_contacts_multiplier +
            else:
                planned_contacts = 0
            # step 2 randomly sample from other neighbours planned contacts
            #other_neighbours = random.sample(other_neighbours, planned_contacts)

            # step 2 by gathering max contacts
            gathering_max_contacts = environment.parameters['gathering_max_contacts']
            if gathering_max_contacts != float('inf'):
                gathering_max_contacts = round(gathering_max_contacts * (1 + (1 - (environment.stringency_index[t] / 100))))
                individual_max_contacts = int(round(gathering_max_contacts * (1 + (1 - agent.compliance))))
            else:
                individual_max_contacts = gathering_max_contacts

            if planned_contacts > individual_max_contacts:
                other_neighbours = random.sample(other_neighbours, individual_max_contacts)
            else:
                other_neighbours = random.sample(other_neighbours, planned_contacts)

            # step 3 combine household neighbours with other neighbours
            neighbours_from_graph = household_neighbours + other_neighbours
            # step 4 find the corresponding agents and add them to a list to infect
            if agent.status in ['i1', 'i2']:
                if agent.compliance == 1.0: #TODO debug, this means an agent with 1.0 compliance will self-isolate
                    neighbours_to_infect = [environment.agents[idx] for idx in household_neighbours]
                else:
                    neighbours_to_infect = [environment.agents[idx] for idx in neighbours_from_graph]
                # step 4 let these agents be infected (with random probability
                physical_distancing_multiplier = 1 - ((1 - environment.parameters["physical_distancing_multiplier"]
                                                       ) * agent.compliance)
                for neighbour in neighbours_to_infect:
                    if neighbour.household_number == agent.household_number and neighbour.district == agent.district:
                        compliance_term_phys_dis = (1 - physical_distancing_multiplier)
                    else:
                        compliance_term_phys_dis = (1 - physical_distancing_multiplier) * agent.compliance

                    if neighbour.status == 's' and np.random.random() < (
                            environment.parameters['probability_transmission'] * (
                            physical_distancing_multiplier + compliance_term_phys_dis)):
                        neighbour.status = 'e'
                        susceptible.remove(neighbour)
                        exposed.append(neighbour)
                        agent.others_infected += 1
                        agent.others_infects_total += 1

            # save compliance to previous compliance
            agent.previous_compliance = agent.compliance
            compliance.append(agent.compliance)

        if data_output == 'network':
            environment.infection_states.append(environment.store_network())
        elif data_output == 'csv':
            environment.write_status_location(t, seed, data_folder)
        elif data_output == 'csv-light':
            # save only the total quantity of agents per category
            for key, quantity in zip(['e', 's', 'i1', 'i2',
                                      'c', 'r', 'd'],
                                     [exposed, susceptible, sick_without_symptoms, sick_with_symptoms,
                                      critical, recovered, dead]):
                environment.infection_quantities[key].append(len(quantity))
            environment.infection_quantities['compliance'].append(np.mean(compliance))

    return environment



# test SABCoM command line application 
import os
import sys

#myPath = os.path.dirname(os.path.abspath(__file__))
#sys.path.insert(0, myPath + '/../')


#from sabcom.__main__ import simulate, initialise


def test_initialise():
    pass
    #runner = cl.CliRunner()
    #default_initialisation_path = os.path.join(os.path.dirname(sys.path[0]), 'example_data/initialisations')
    #result = runner.invoke(initialise, [3, default_initialisation_path])
    #assert result.exit_code == 0
	
	
def test_initialise():
    pass
	#runner = cl.CliRunner()
    #default_output_path = os.path.join(os.path.dirname(sys.path[0]), 'example_data/output_data')
    #result = runner.invoke(simulate, [3, default_output_path])
    #assert result.exit_code == 0



class Agent:
    def __init__(self, name, status, district, age_group,
                 informality, number_contacts, district_to_travel_to, compliance):
        """
        This method initialises an agent and its properties.

        Agents have several properties, that can either be state variables (if they change),
        or static parameters. There is a distinction between agent specific parameters
        and parameters that are the same for all agents and could thus be seen
        as global parameters.

        The following inputs are used to initialise the agent
        :param name: unique agent identifier , integer
        :param status: initial disease status, string
        :param coordinates: the geographical coordinates of where the agent lives, tuple (float, float)
        :param district: the unique code / identifier of the district, int
        :param age_group: the age group of the agent, float ([age_0_10', 'age_10_20', 'age_20_30', 'age_30_40', 'age_40_50',
              'age_50_60', 'age_60_70', 'age_70_80', 'age_80_plus')
        :param informality: a percentage indicating how 'informal' the district the agent lives in is, float (0,1)
        :param number_contacts: the amount of trips the agent will undertake on a daily basis
        """
        # state variables
        self.status = status

        # agent specific parameters
        self.name = name
        self.district = district
        self.age_group = age_group

        # agent specific variables
        self.compliance = compliance  # TODO debug
        self.previous_compliance = compliance

        # implementation variables
        self.sick_days = 0
        self.asymptomatic_days = 0
        self.incubation_days = 0
        self.critical_days = 0
        self.exposed_days = 0
        self.days_recovered = 0
        self.others_infected = 0
        self.others_infects_total = 0
        self.travel_neighbours = []

        # implementation parameters
        self.num_contacts = number_contacts
        self.household_number = None

        # agent specific parameters that depend on other parameters
        self.district_to_travel_to = district_to_travel_to
        self.informality = informality

    def __repr__(self):
        """
        :return: String representation of the trader
        """
        return self.status + ' Agent' + str(self.name)



from setuptools import setup, find_packages
from io import open
from os import path

import pathlib

# credits to Oyetoke Tobi Emmanuel for providing the template for this file
# This is the directory containing this path
HERE = pathlib.Path(__file__).parent

# Read in the readme file
README = (HERE / "README.md").read_text()

# this will automatically captured required modules for install_requires in requirements.txt
with open(path.join(HERE, 'requirements.txt'), encoding='utf-8') as f:
    all_reqs = f.read().split('\n')

install_requires = [x.strip() for x in all_reqs if ('git+' not in x) and (
    not x.startswith('#')) and (not x.startswith('-'))]
dependency_links = [x.strip().replace('git+', '') for x in all_reqs \
                    if 'git+' not in x]
setup (
 name = 'sabcom',
 description = 'SABCOM is an open source, easy-to-use, spatial network, agent-based, model that can be used to simulate the effects of different lockdown policy measures on the spread of the Covid-19 virus in several (South African) cities.',
 version = '0.10.2a',
 packages = find_packages(), # list of all packages
 #install_requires = install_requires,
 python_requires='>=3.6', # any python greater than 3.6
 entry_points='''
        [console_scripts]
        sabcom=sabcom.__main__:main
    ''',
 author="Joeri Schasfoort",
 keyword="sabcom, covid-19, South Africa, simulation, Coronavirus",
 long_description=README,
 long_description_content_type="text/markdown",
 license='MIT',
 url='https://github.com/blackrhinoabm/sabcom',
 download_url='https://github.com/blackrhinoabm/sabcom/archive/1.0.0.tar.gz',
  #dependency_links=dependency_links,
  author_email='joeri.schasfoort@uct.ac.za',
  classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ]
)
